@@ -30,6 +30,7 @@ typedef struct
 	GtkTextMark             *mark;
 	GtkSourceAssistantChild *child;
 	guint                    reposition_handler;
+	GtkPositionType          preferred_position;
 } GtkSourceAssistantPrivate;
 
 static void buildable_iface_init (GtkBuildableIface *iface);
@@ -135,16 +136,66 @@ _gtk_source_assistant_update_position (GtkSourceAssistant *assistant)
 		GdkRectangle rect;
 		int old_x, old_y;
 		int x, y;
+		GtkRoot *root;
 
 		gtk_text_view_get_visible_rect (GTK_TEXT_VIEW (parent), &visible_rect);
 
-		_gtk_source_assistant_get_offset (assistant, &x, &y);
 		_gtk_source_assistant_get_target_location (assistant, &rect);
 
 		rect.x -= visible_rect.x;
 		rect.y -= visible_rect.y;
 		rect.x += get_gutter_width (GTK_SOURCE_VIEW (parent));
 
+		if (rect.x < 0 || rect.x + rect.width > visible_rect.width ||
+		    rect.y < 0 || rect.y + rect.height > visible_rect.height)
+		{
+			gtk_widget_set_visible (GTK_WIDGET (assistant), FALSE);
+			return FALSE;
+		}
+
+		root = gtk_widget_get_root (parent);
+
+		if (GTK_IS_WINDOW (root))
+		{
+			graphene_point_t point;
+			graphene_point_t root_point;
+
+			point.x = rect.x;
+			point.y = rect.y;
+
+			if (gtk_widget_compute_point (parent, GTK_WIDGET (root), &point, &root_point))
+			{
+				GtkRequisition natural_height;
+				GtkPositionType position;
+				int window_height;
+
+				gtk_widget_get_preferred_size (GTK_WIDGET(assistant), NULL, &natural_height);
+
+				window_height = gtk_widget_get_height (GTK_WIDGET (root));
+
+				position = priv->preferred_position;
+
+				if (position == GTK_POS_BOTTOM &&
+				    (root_point.y + rect.height + natural_height.height) > window_height)
+				{
+					position = GTK_POS_TOP;
+					gtk_widget_add_css_class (GTK_WIDGET (assistant), "above-line");
+				}
+				else
+				{
+					gtk_widget_remove_css_class (GTK_WIDGET (assistant), "above-line");
+				}
+
+				if (gtk_popover_get_position (GTK_POPOVER (assistant)) != position)
+				{
+					gtk_popover_set_position (GTK_POPOVER (assistant), position);
+					changed = TRUE;
+				}
+			}
+		}
+
+		_gtk_source_assistant_get_offset (assistant, &x, &y);
+
 		gtk_popover_get_offset (GTK_POPOVER (assistant), &old_x, &old_y);
 
 		if (old_x != x || old_y != y)
@@ -209,6 +260,8 @@ _gtk_source_assistant_real_get_offset (GtkSourceAssistant *assistant,
                                        int                *x,
                                        int                *y)
 {
+	G_GNUC_BEGIN_IGNORE_DEPRECATIONS
+
 	GtkStyleContext *style_context;
 	GtkBorder margin;
 	GtkPositionType pos;
@@ -232,6 +285,8 @@ _gtk_source_assistant_real_get_offset (GtkSourceAssistant *assistant,
 	{
 		*y = margin.bottom - 1;
 	}
+
+	G_GNUC_END_IGNORE_DEPRECATIONS
 }
 
 static void
@@ -275,12 +330,13 @@ _gtk_source_assistant_init (GtkSourceAssistant *self)
 
 	gtk_widget_set_halign (GTK_WIDGET (self), GTK_ALIGN_START);
 	gtk_widget_set_valign (GTK_WIDGET (self), GTK_ALIGN_START);
-	gtk_popover_set_position (GTK_POPOVER (self), GTK_POS_BOTTOM);
 	gtk_popover_set_has_arrow (GTK_POPOVER (self), FALSE);
 	gtk_popover_set_autohide (GTK_POPOVER (self), TRUE);
 
 	priv->child = _gtk_source_assistant_child_new ();
 	gtk_popover_set_child (GTK_POPOVER (self), GTK_WIDGET (priv->child));
+
+	priv->preferred_position = GTK_POS_BOTTOM;
 }
 
 GtkSourceAssistant *
@@ -430,3 +486,12 @@ _gtk_source_assistant_destroy (GtkSourceAssistant *self)
 			   G_OBJECT_TYPE_NAME (parent));
 	}
 }
+
+void
+_gtk_source_assistant_set_pref_position (GtkSourceAssistant *self,
+                                         GtkPositionType     position)
+{
+	GtkSourceAssistantPrivate *priv = _gtk_source_assistant_get_instance_private (self);
+
+	priv->preferred_position = position;
+}
