@@ -43,3 +43,28 @@ static void
 gtk_source_completion_proposal_default_init (GtkSourceCompletionProposalInterface *iface)
 {
 }
+
+/**
+ * gtk_source_completion_proposal_get_typed_text:
+ * @proposal: a #GtkSourceCompletionProposal
+ *
+ * Gets the typed-text for the proposal, if supported by the implementation.
+ *
+ * Implementing this virtual-function is optional, but can be useful to allow
+ * external tooling to compare results.
+ *
+ * Returns: (transfer full) (nullable): a newly allocated string, or %NULL
+ *
+ * Since: 5.6
+ */
+char *
+gtk_source_completion_proposal_get_typed_text (GtkSourceCompletionProposal *proposal)
+{
+	GtkSourceCompletionProposalInterface *iface;
+
+	g_return_val_if_fail (GTK_SOURCE_IS_COMPLETION_PROPOSAL (proposal), NULL);
+
+	iface = GTK_SOURCE_COMPLETION_PROPOSAL_GET_IFACE (proposal);
+
+	return iface->get_typed_text ? iface->get_typed_text (proposal) : NULL;
+}
