@@ -23,6 +23,8 @@
 
 #include <string.h>
 
+#include "gtksourceannotation-private.h"
+#include "gtksourceannotationprovider.h"
 #include "gtksourceassistant-private.h"
 #include "gtksourcehoverassistant-private.h"
 #include "gtksourcehovercontext-private.h"
@@ -312,6 +314,8 @@ static void
 gtk_source_hover_assistant_get_target_location (GtkSourceAssistant *assistant,
                                                 GdkRectangle       *rect)
 {
+	G_GNUC_BEGIN_IGNORE_DEPRECATIONS
+
 	GtkSourceHoverAssistant *self = GTK_SOURCE_HOVER_ASSISTANT (assistant);
 	GtkStyleContext *style_context;
 	GtkBorder padding;
@@ -322,6 +326,8 @@ gtk_source_hover_assistant_get_target_location (GtkSourceAssistant *assistant,
 	gtk_style_context_get_padding (style_context, &padding);
 
 	rect->x -= padding.left;
+
+	G_GNUC_END_IGNORE_DEPRECATIONS
 }
 
 static void
@@ -394,6 +400,8 @@ gtk_source_hover_assistant_init (GtkSourceHoverAssistant *self)
 
 	gtk_widget_add_css_class (GTK_WIDGET (self), "hover-assistant");
 
+	_gtk_source_assistant_set_pref_position (GTK_SOURCE_ASSISTANT (self), GTK_POS_TOP);
+
 	gtk_popover_set_autohide (GTK_POPOVER (self), FALSE);
 	gtk_popover_set_position (GTK_POPOVER (self), GTK_POS_TOP);
 
@@ -536,6 +544,83 @@ _gtk_source_hover_assistant_display (GtkSourceHoverAssistant  *self,
 	g_object_unref (context);
 }
 
+static void
+gtk_source_hover_assistant_populate_annotation_cb (GObject      *object,
+                                                   GAsyncResult *result,
+                                                   gpointer      user_data)
+{
+	GtkSourceAnnotationProvider *provider = (GtkSourceAnnotationProvider *)object;
+	GtkSourceHoverAssistant *self = user_data;
+	GError *error = NULL;
+
+	g_assert (GTK_SOURCE_IS_ANNOTATION_PROVIDER (provider));
+	g_assert (G_IS_ASYNC_RESULT (result));
+	g_assert (GTK_SOURCE_IS_HOVER_ASSISTANT (self));
+
+	if (gtk_source_annotation_provider_populate_hover_finish (provider, result, &error))
+	{
+		if (!self->disposed)
+		{
+			GtkWidget *parent = gtk_widget_get_parent (GTK_WIDGET (self));
+			gboolean mapped = parent && gtk_widget_get_mapped (parent);
+			gboolean empty = _gtk_source_hover_display_is_empty (self->display);
+
+			gtk_widget_set_visible (GTK_WIDGET (self), mapped && !empty);
+		}
+	}
+
+	g_clear_object (&self);
+	g_clear_error (&error);
+}
+
+void
+_gtk_source_hover_assistant_display_annotation (GtkSourceHoverAssistant     *self,
+                                                GtkSourceAnnotationProvider *provider,
+                                                GtkSourceAnnotation         *annotation)
+{
+	GtkSourceView *view;
+	GdkRectangle location_rect;
+	GdkRectangle visible_rect;
+	int buffer_x, buffer_y;
+
+	memset (&self->hovered_at, 0, sizeof self->hovered_at);
+
+	_gtk_source_annotation_get_rect (annotation, &location_rect);
+
+	view = GTK_SOURCE_VIEW (gtk_widget_get_parent (GTK_WIDGET (self)));
+
+	gtk_text_view_window_to_buffer_coords (GTK_TEXT_VIEW (view),
+	                                       GTK_TEXT_WINDOW_TEXT,
+	                                       location_rect.x,
+	                                       location_rect.y,
+	                                       &buffer_x,
+	                                       &buffer_y);
+	location_rect.x = buffer_x;
+	location_rect.y = buffer_y;
+
+	gtk_text_view_get_visible_rect (GTK_TEXT_VIEW (view), &visible_rect);
+
+	if (!gdk_rectangle_intersect (&location_rect, &visible_rect, &location_rect))
+	{
+		gtk_widget_set_visible (GTK_WIDGET (self), FALSE);
+		return;
+	}
+
+	self->hovered_at = location_rect;
+
+	_gtk_source_hover_display_clear (self->display);
+
+	g_clear_object (&self->cancellable);
+	self->cancellable = g_cancellable_new ();
+
+	gtk_source_annotation_provider_populate_hover_async (provider,
+	                                                     annotation,
+	                                                     self->display,
+	                                                     self->cancellable,
+	                                                     gtk_source_hover_assistant_populate_annotation_cb,
+	                                                     g_object_ref (self));
+}
+
 void
 _gtk_source_hover_assistant_dismiss (GtkSourceHoverAssistant *self)
 {
