@@ -24,11 +24,11 @@
 /* GtkTextIter functions. Contains forward/backward functions for word
  * movements, with custom word boundaries that are used for word selection
  * (double-click) and cursor movements (Ctrl+left, Ctrl+right, etc).  The
- * initial idea was to use those word boundaries directly in GTK+, for all text
+ * initial idea was to use those word boundaries directly in GTK, for all text
  * widgets. But in the end only the GtkTextView::extend-selection signal has
  * been added to be able to customize the boundaries for double- and
  * triple-click (the ::move-cursor and ::delete-from-cursor signals were already
- * present to customize boundaries for cursor movements). The GTK+ developers
+ * present to customize boundaries for cursor movements). The GTK developers
  * didn't want to change the word boundaries for text widgets. More information:
  * https://mail.gnome.org/archives/gtk-devel-list/2014-September/msg00019.html
  * https://bugzilla.gnome.org/show_bug.cgi?id=111503
@@ -587,6 +587,30 @@ _gtk_source_iter_inside_word (const GtkTextIter *iter)
 {
 	GtkTextIter prev_word_start;
 	GtkTextIter word_end;
+	gunichar ch;
+
+	/* Short-circuit by looking at our current character
+	 * and the previous character. This can save considerable
+	 * time on complex layouts such as JSON formatted latin
+	 * form text.
+	 */
+	ch = gtk_text_iter_get_char (iter);
+	if (g_unichar_isalnum (ch) || ch == '_')
+	{
+		GtkTextIter prev = *iter;
+
+		if (!gtk_text_iter_backward_char (&prev))
+		{
+			return TRUE;
+		}
+
+		ch = gtk_text_iter_get_char (iter);
+
+		if (g_unichar_isalnum (ch) || ch == '_')
+		{
+			return TRUE;
+		}
+	}
 
 	if (_gtk_source_iter_starts_word (iter))
 	{
@@ -612,7 +636,7 @@ _gtk_source_iter_extend_selection_word (const GtkTextIter *location,
 					GtkTextIter       *start,
 					GtkTextIter       *end)
 {
-	/* Exactly the same algorithm as in GTK+, but with our custom word
+	/* Exactly the same algorithm as in GTK, but with our custom word
 	 * boundaries.
 	 */
 	*start = *location;
