@@ -23,7 +23,7 @@
 
 G_BEGIN_DECLS
 
-G_GNUC_INTERNAL
 GtkSourceLanguageManager *_gtk_source_language_manager_peek_default (void);
+void                      _gtk_source_language_manager_set_rng_file (const char *rng_file);
 
 G_END_DECLS
