@@ -588,7 +588,7 @@ pick_langs_for_filename (GtkSourceLanguageManager *lm,
 			/* FIXME g_pattern_match is wrong: there are no '[...]'
 			 * character ranges and '*' and '?' can not be escaped
 			 * to include them literally in a pattern.  */
-			if (g_pattern_match_simple (*gptr, filename_utf8))
+			if (**gptr != '\0' && g_pattern_match_simple (*gptr, filename_utf8))
 			{
 				langs = g_slist_prepend (langs, lang);
 				break;
@@ -599,7 +599,8 @@ pick_langs_for_filename (GtkSourceLanguageManager *lm,
 	}
 
 	g_free (filename_utf8);
-	return langs;
+
+	return g_slist_reverse (langs);
 }
 
 static GtkSourceLanguage *
@@ -723,6 +724,11 @@ pick_lang_for_mime_type (GtkSourceLanguageManager *lm,
  * lang = gtk_source_language_manager_guess_language (manager, filename, NULL);
  * gtk_source_buffer_set_language (buffer, lang);
  * ```
+ * ```python
+ * manager = GtkSource.LanguageManager.get_default()
+ * language = manager.guess_language(filename=filename, content_type=None)
+ * buffer.set_language(language=language)
+ * ```
  *
  * or
  *
@@ -745,6 +751,15 @@ pick_lang_for_mime_type (GtkSourceLanguageManager *lm,
  *
  * g_free (content_type);
  * ```
+ * ```python
+ * content_type, uncertain = Gio.content_type_guess(filename=filename, data=None)
+ * if uncertain:
+ *     content_type = None
+ *
+ * manager = GtkSource.LanguageManager.get_default()
+ * language = manager.guess_language(filename=filename, content_type=content_type)
+ * buffer.set_language(language=language)
+ * ```
  *
  * etc. Use [method@Language.get_mime_types] and [method@Language.get_globs]
  * if you need full control over file -> language mapping.
