@@ -22,14 +22,13 @@
 #include "gtksourcesearchsettings.h"
 
 /**
- * SECTION:searchsettings
- * @Short_description: Search settings
- * @Title: GtkSourceSearchSettings
- * @See_also: #GtkSourceSearchContext
+ * GtkSourceSearchSettings:
+ * 
+ * Search settings.
  *
- * A #GtkSourceSearchSettings object represents the settings of a search. The
+ * A `GtkSourceSearchSettings` object represents the settings of a search. The
  * search settings can be associated with one or several
- * #GtkSourceSearchContext<!-- -->s.
+ * [class@SearchContext]s.
  */
 
 enum
@@ -40,6 +39,7 @@ enum
 	PROP_AT_WORD_BOUNDARIES,
 	PROP_WRAP_AROUND,
 	PROP_REGEX_ENABLED,
+	PROP_VISIBLE_ONLY,
 	N_PROPS
 };
 
@@ -50,6 +50,7 @@ typedef struct
 	guint at_word_boundaries : 1;
 	guint wrap_around : 1;
 	guint regex_enabled : 1;
+	guint visible_only : 1;
 } GtkSourceSearchSettingsPrivate;
 
 G_DEFINE_TYPE_WITH_PRIVATE (GtkSourceSearchSettings, gtk_source_search_settings, G_TYPE_OBJECT)
@@ -98,6 +99,10 @@ gtk_source_search_settings_get_property (GObject    *object,
 			g_value_set_boolean (value, priv->regex_enabled);
 			break;
 
+		case PROP_VISIBLE_ONLY:
+			g_value_set_boolean (value, priv->visible_only);
+			break;
+
 		default:
 			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 			break;
@@ -135,6 +140,10 @@ gtk_source_search_settings_set_property (GObject      *object,
 			priv->regex_enabled = g_value_get_boolean (value);
 			break;
 
+		case PROP_VISIBLE_ONLY:
+			priv->visible_only = g_value_get_boolean (value);
+			break;
+
 		default:
 			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 			break;
@@ -153,11 +162,10 @@ gtk_source_search_settings_class_init (GtkSourceSearchSettingsClass *klass)
 	/**
 	 * GtkSourceSearchSettings:search-text:
 	 *
-	 * A search string, or %NULL if the search is disabled. If the regular
-	 * expression search is enabled, #GtkSourceSearchSettings:search-text is
+	 * A search string, or %NULL if the search is disabled. 
+	 * 
+	 * If the regular expression search is enabled, [property@SearchSettings:search-text] is
 	 * the pattern.
-	 *
-	 * Since: 3.10
 	 */
 	properties[PROP_SEARCH_TEXT] =
 		g_param_spec_string ("search-text",
@@ -170,8 +178,6 @@ gtk_source_search_settings_class_init (GtkSourceSearchSettingsClass *klass)
 	 * GtkSourceSearchSettings:case-sensitive:
 	 *
 	 * Whether the search is case sensitive.
-	 *
-	 * Since: 3.10
 	 */
 	properties[PROP_CASE_SENSITIVE] =
 		g_param_spec_boolean ("case-sensitive",
@@ -185,8 +191,6 @@ gtk_source_search_settings_class_init (GtkSourceSearchSettingsClass *klass)
 	 *
 	 * If %TRUE, a search match must start and end a word. The match can
 	 * span multiple words.
-	 *
-	 * Since: 3.10
 	 */
 	properties[PROP_AT_WORD_BOUNDARIES] =
 		g_param_spec_boolean ("at-word-boundaries",
@@ -201,8 +205,6 @@ gtk_source_search_settings_class_init (GtkSourceSearchSettingsClass *klass)
 	 * For a forward search, continue at the beginning of the buffer if no
 	 * search occurrence is found. For a backward search, continue at the
 	 * end of the buffer.
-	 *
-	 * Since: 3.10
 	 */
 	properties[PROP_WRAP_AROUND] =
 		g_param_spec_boolean ("wrap-around",
@@ -215,9 +217,7 @@ gtk_source_search_settings_class_init (GtkSourceSearchSettingsClass *klass)
 	 * GtkSourceSearchSettings:regex-enabled:
 	 *
 	 * Search by regular expressions with
-	 * #GtkSourceSearchSettings:search-text as the pattern.
-	 *
-	 * Since: 3.10
+	 * [property@SearchSettings:search-text] as the pattern.
 	 */
 	properties[PROP_REGEX_ENABLED] =
 		g_param_spec_boolean ("regex-enabled",
@@ -226,6 +226,21 @@ gtk_source_search_settings_class_init (GtkSourceSearchSettingsClass *klass)
 				      FALSE,
 				      G_PARAM_READWRITE | G_PARAM_CONSTRUCT | G_PARAM_STATIC_STRINGS);
 
+	/**
+	 * GtkSourceSearchSettings:visible-only:
+	 *
+	 * Exclude invisible text from the search.
+	 * A search match may have invisible text interspersed.
+	 *
+	 * Since: 5.12
+	 */
+	properties[PROP_VISIBLE_ONLY] =
+		g_param_spec_boolean ("visible-only",
+				      "Visible only",
+				      "Whether to exclude invisible text from the search",
+				      TRUE,
+				      G_PARAM_READWRITE | G_PARAM_CONSTRUCT | G_PARAM_STATIC_STRINGS);
+
 	g_object_class_install_properties (object_class, N_PROPS, properties);
 }
 
@@ -240,7 +255,6 @@ gtk_source_search_settings_init (GtkSourceSearchSettings *self)
  * Creates a new search settings object.
  *
  * Returns: a new search settings object.
- * Since: 3.10
  */
 GtkSourceSearchSettings *
 gtk_source_search_settings_new (void)
@@ -253,14 +267,13 @@ gtk_source_search_settings_new (void)
  * @settings: a #GtkSourceSearchSettings.
  * @search_text: (nullable): the nul-terminated text to search, or %NULL to disable the search.
  *
- * Sets the text to search. If @search_text is %NULL or is empty, the search
- * will be disabled. A copy of @search_text will be made, so you can safely free
- * @search_text after a call to this function.
+ * Sets the text to search. 
+ * 
+ * If @search_text is %NULL or is empty, the search will be disabled. A copy of @search_text 
+ * will be made, so you can safely free @search_text after a call to this function.
  *
- * You may be interested to call gtk_source_utils_unescape_search_text() before
+ * You may be interested to call [func@utils_unescape_search_text] before
  * this function.
- *
- * Since: 3.10
  */
 void
 gtk_source_search_settings_set_search_text (GtkSourceSearchSettings *settings,
@@ -296,13 +309,14 @@ gtk_source_search_settings_set_search_text (GtkSourceSearchSettings *settings,
  * gtk_source_search_settings_get_search_text:
  * @settings: a #GtkSourceSearchSettings.
  *
- * Gets the text to search. The return value must not be freed.
+ * Gets the text to search. 
+ * 
+ * The return value must not be freed.
  *
- * You may be interested to call gtk_source_utils_escape_search_text() after
+ * You may be interested to call [func@utils_escape_search_text] after
  * this function.
  *
  * Returns: (nullable): the text to search, or %NULL if the search is disabled.
- * Since: 3.10
  */
 const gchar *
 gtk_source_search_settings_get_search_text (GtkSourceSearchSettings *settings)
@@ -320,8 +334,6 @@ gtk_source_search_settings_get_search_text (GtkSourceSearchSettings *settings)
  * @case_sensitive: the setting.
  *
  * Enables or disables the case sensitivity for the search.
- *
- * Since: 3.10
  */
 void
 gtk_source_search_settings_set_case_sensitive (GtkSourceSearchSettings *settings,
@@ -345,7 +357,6 @@ gtk_source_search_settings_set_case_sensitive (GtkSourceSearchSettings *settings
  * @settings: a #GtkSourceSearchSettings.
  *
  * Returns: whether the search is case sensitive.
- * Since: 3.10
  */
 gboolean
 gtk_source_search_settings_get_case_sensitive (GtkSourceSearchSettings *settings)
@@ -362,12 +373,11 @@ gtk_source_search_settings_get_case_sensitive (GtkSourceSearchSettings *settings
  * @settings: a #GtkSourceSearchSettings.
  * @at_word_boundaries: the setting.
  *
- * Change whether the search is done at word boundaries. If @at_word_boundaries
- * is %TRUE, a search match must start and end a word. The match can span
- * multiple words. See also gtk_text_iter_starts_word() and
- * gtk_text_iter_ends_word().
- *
- * Since: 3.10
+ * Change whether the search is done at word boundaries. 
+ * 
+ * If @at_word_boundaries is %TRUE, a search match must start and end a word.
+ * The match can span multiple words. See also [method@Gtk.TextIter.starts_word] and
+ * [method@Gtk.TextIter.ends_word].
  */
 void
 gtk_source_search_settings_set_at_word_boundaries (GtkSourceSearchSettings *settings,
@@ -391,7 +401,6 @@ gtk_source_search_settings_set_at_word_boundaries (GtkSourceSearchSettings *sett
  * @settings: a #GtkSourceSearchSettings.
  *
  * Returns: whether to search at word boundaries.
- * Since: 3.10
  */
 gboolean
 gtk_source_search_settings_get_at_word_boundaries (GtkSourceSearchSettings *settings)
@@ -408,12 +417,11 @@ gtk_source_search_settings_get_at_word_boundaries (GtkSourceSearchSettings *sett
  * @settings: a #GtkSourceSearchSettings.
  * @wrap_around: the setting.
  *
- * Enables or disables the wrap around search. If @wrap_around is %TRUE, the
- * forward search continues at the beginning of the buffer if no search
- * occurrences are found. Similarly, the backward search continues to search at
+ * Enables or disables the wrap around search. 
+ * 
+ * If @wrap_around is %TRUE, the forward search continues at the beginning of the buffer
+ * if no search occurrences are found. Similarly, the backward search continues to search at
  * the end of the buffer.
- *
- * Since: 3.10
  */
 void
 gtk_source_search_settings_set_wrap_around (GtkSourceSearchSettings *settings,
@@ -437,7 +445,6 @@ gtk_source_search_settings_set_wrap_around (GtkSourceSearchSettings *settings,
  * @settings: a #GtkSourceSearchSettings.
  *
  * Returns: whether to wrap around the search.
- * Since: 3.10
  */
 gboolean
 gtk_source_search_settings_get_wrap_around (GtkSourceSearchSettings *settings)
@@ -455,14 +462,13 @@ gtk_source_search_settings_get_wrap_around (GtkSourceSearchSettings *settings)
  * @regex_enabled: the setting.
  *
  * Enables or disables whether to search by regular expressions.
- * If enabled, the #GtkSourceSearchSettings:search-text property contains the
+ *
+ * If enabled, the [property@SearchSettings:search-text] property contains the
  * pattern of the regular expression.
  *
- * #GtkSourceSearchContext uses #GRegex when regex search is enabled. See the
+ * [class@SearchContext] uses #GRegex when regex search is enabled. See the
  * [Regular expression syntax](https://developer.gnome.org/glib/stable/glib-regex-syntax.html)
  * page in the GLib reference manual.
- *
- * Since: 3.10
  */
 void
 gtk_source_search_settings_set_regex_enabled (GtkSourceSearchSettings *settings,
@@ -486,7 +492,6 @@ gtk_source_search_settings_set_regex_enabled (GtkSourceSearchSettings *settings,
  * @settings: a #GtkSourceSearchSettings.
  *
  * Returns: whether to search by regular expressions.
- * Since: 3.10
  */
 gboolean
 gtk_source_search_settings_get_regex_enabled (GtkSourceSearchSettings *settings)
@@ -497,3 +502,50 @@ gtk_source_search_settings_get_regex_enabled (GtkSourceSearchSettings *settings)
 
 	return priv->regex_enabled;
 }
+
+/**
+ * gtk_source_search_settings_set_visible_only:
+ * @settings: a #GtkSourceSearchSettings.
+ * @visible_only: the setting.
+ *
+ * Enables or disables whether to exclude invisible text from the search.
+ *
+ * If enabled, only visible text will be searched.
+ * A search match may have invisible text interspersed.
+ *
+ * Since: 5.12
+ */
+void
+gtk_source_search_settings_set_visible_only (GtkSourceSearchSettings *settings,
+                                             gboolean                 visible_only)
+{
+	GtkSourceSearchSettingsPrivate *priv = gtk_source_search_settings_get_instance_private (settings);
+
+	g_return_if_fail (GTK_SOURCE_IS_SEARCH_SETTINGS (settings));
+
+	visible_only = visible_only != FALSE;
+
+	if (priv->visible_only != visible_only)
+	{
+		priv->visible_only = visible_only;
+		g_object_notify_by_pspec (G_OBJECT (settings), properties[PROP_VISIBLE_ONLY]);
+	}
+}
+
+/**
+ * gtk_source_search_settings_get_visible_only:
+ * @settings: a #GtkSourceSearchSettings.
+ *
+ * Returns: whether to exclude invisible text from the search.
+ *
+ * Since: 5.12
+ */
+gboolean
+gtk_source_search_settings_get_visible_only (GtkSourceSearchSettings *settings)
+{
+	GtkSourceSearchSettingsPrivate *priv = gtk_source_search_settings_get_instance_private (settings);
+
+	g_return_val_if_fail (GTK_SOURCE_IS_SEARCH_SETTINGS (settings), FALSE);
+
+	return priv->visible_only;
+}
