@@ -31,6 +31,7 @@
 #include "gtksourceview.h"
 #include "gtksourcelanguage-private.h"
 #include "gtksourcetrace.h"
+#include "gtksourceutils-private.h"
 
 /**
  * GtkSourceStyleScheme:
@@ -66,6 +67,10 @@
 #define STYLE_SNIPPET_FOCUS		"snippet-focus"
 #define STYLE_BACKGROUND_PATTERN	"background-pattern"
 
+#define STYLE_WARNING                   "diff:changed-line"
+#define STYLE_ERROR                     "diff:removed-line"
+#define STYLE_ACCENT                    "diff:added-line"
+
 #define STYLE_SCHEME_VERSION		"1.0"
 
 #define DEFAULT_STYLE_SCHEME		"classic"
@@ -747,6 +752,48 @@ _gtk_source_style_scheme_get_text_color (GtkSourceStyleScheme *scheme,
 	return get_color (style, TRUE, color);
 }
 
+gboolean
+_gtk_source_style_scheme_get_warning_color (GtkSourceStyleScheme *scheme,
+                                            GdkRGBA              *color)
+{
+	GtkSourceStyle *style;
+
+	g_return_val_if_fail (GTK_SOURCE_IS_STYLE_SCHEME (scheme), FALSE);
+	g_return_val_if_fail (color != NULL, FALSE);
+
+	style = gtk_source_style_scheme_get_style (scheme, STYLE_WARNING);
+
+	return get_color (style, TRUE, color);
+}
+
+gboolean
+_gtk_source_style_scheme_get_error_color (GtkSourceStyleScheme *scheme,
+                                          GdkRGBA              *color)
+{
+	GtkSourceStyle *style;
+
+	g_return_val_if_fail (GTK_SOURCE_IS_STYLE_SCHEME (scheme), FALSE);
+	g_return_val_if_fail (color != NULL, FALSE);
+
+	style = gtk_source_style_scheme_get_style (scheme, STYLE_ERROR);
+
+	return get_color (style, TRUE, color);
+}
+
+gboolean
+_gtk_source_style_scheme_get_accent_color (GtkSourceStyleScheme *scheme,
+                                           GdkRGBA              *color)
+{
+	GtkSourceStyle *style;
+
+	g_return_val_if_fail (GTK_SOURCE_IS_STYLE_SCHEME (scheme), FALSE);
+	g_return_val_if_fail (color != NULL, FALSE);
+
+	style = gtk_source_style_scheme_get_style (scheme, STYLE_ACCENT);
+
+	return get_color (style, TRUE, color);
+}
+
 static void
 apply_css_style_cursors (GtkSourceStyleScheme *scheme,
                          GString              *css)
@@ -816,8 +863,6 @@ void
 _gtk_source_style_scheme_apply (GtkSourceStyleScheme *scheme,
                                 GtkWidget            *widget)
 {
-	GtkStyleContext *context;
-
 	g_return_if_fail (GTK_SOURCE_IS_STYLE_SCHEME (scheme));
 	g_return_if_fail (!widget || GTK_IS_WIDGET (widget));
 
@@ -826,10 +871,9 @@ _gtk_source_style_scheme_apply (GtkSourceStyleScheme *scheme,
 		return;
 	}
 
-	context = gtk_widget_get_style_context (widget);
-	gtk_style_context_add_provider (context,
-	                                GTK_STYLE_PROVIDER (scheme->css_provider),
-	                                GTK_SOURCE_STYLE_PROVIDER_PRIORITY);
+	_gtk_source_widget_add_css_provider (widget,
+	                                     scheme->css_provider,
+	                                     GTK_SOURCE_STYLE_PROVIDER_PRIORITY);
 }
 
 /**
@@ -843,8 +887,6 @@ void
 _gtk_source_style_scheme_unapply (GtkSourceStyleScheme *scheme,
                                   GtkWidget            *widget)
 {
-	GtkStyleContext *context;
-
 	g_return_if_fail (GTK_SOURCE_IS_STYLE_SCHEME (scheme));
 	g_return_if_fail (!widget || GTK_IS_WIDGET (widget));
 
@@ -853,9 +895,7 @@ _gtk_source_style_scheme_unapply (GtkSourceStyleScheme *scheme,
 		return;
 	}
 
-	context = gtk_widget_get_style_context (widget);
-	gtk_style_context_remove_provider (context,
-	                                   GTK_STYLE_PROVIDER (scheme->css_provider));
+	_gtk_source_widget_remove_css_provider (widget, scheme->css_provider);
 }
 
 /* --- PARSER ---------------------------------------------------------------- */
