@@ -24,6 +24,7 @@
 #include "gtksourcestyleschememanager-private.h"
 #include "gtksourcestylescheme-private.h"
 #include "gtksourceutils-private.h"
+#include "gtksourcetrace.h"
 
 /**
  * GtkSourceStyleSchemeManager:
@@ -176,7 +177,7 @@ gtk_source_style_scheme_manager_init (GtkSourceStyleSchemeManager *mgr)
  *
  * Creates a new style manager.
  *
- * If you do not need more than one style manager 
+ * If you do not need more than one style manager
  * then use [func@StyleSchemeManager.get_default] instead.
  *
  * Returns: a new #GtkSourceStyleSchemeManager.
@@ -344,6 +345,8 @@ reload_if_needed (GtkSourceStyleSchemeManager *mgr)
 	if (!mgr->need_reload)
 		return;
 
+	GTK_SOURCE_PROFILER_BEGIN_MARK
+
 	schemes_hash = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, g_object_unref);
 
 	files = _gtk_source_utils_get_file_list ((gchar **)gtk_source_style_scheme_manager_get_search_path (mgr),
@@ -389,6 +392,8 @@ reload_if_needed (GtkSourceStyleSchemeManager *mgr)
 
 	mgr->ids = schemes_list_to_ids (schemes);
 	g_slist_free (schemes);
+
+	GTK_SOURCE_PROFILER_END_MARK ("StyleSchemeManager", "reload");
 }
 
 static void
@@ -410,6 +415,10 @@ notify_search_path (GtkSourceStyleSchemeManager *mgr)
  * style scheme files.
  *
  * If @path is %NULL, the search path is reset to default.
+ *
+ * Since GtkSourceView 5.16 this function will allow you to provide
+ * paths in the form of "resource:///" URIs to embedded `GResource`s.
+ * They must contain the path of a directory within the `GResource`.
  */
 void
 gtk_source_style_scheme_manager_set_search_path (GtkSourceStyleSchemeManager *manager,
@@ -457,9 +466,9 @@ gtk_source_style_scheme_manager_append_search_path (GtkSourceStyleSchemeManager
 
 	len = g_strv_length (manager->search_path);
 
-	manager->search_path = g_renew (gchar *,
-					      manager->search_path,
-					      len + 2); /* old path + new entry + NULL */
+	manager->search_path = g_renew (char *,
+	                                manager->search_path,
+	                                len + 2); /* old path + new entry + NULL */
 
 	manager->search_path[len] = g_strdup (path);
 	manager->search_path[len + 1] = NULL;
@@ -532,8 +541,8 @@ gtk_source_style_scheme_manager_get_search_path (GtkSourceStyleSchemeManager *ma
  * @manager: a #GtkSourceStyleSchemeManager.
  *
  * Mark any currently cached information about the available style schems
- * as invalid. 
- * 
+ * as invalid.
+ *
  * All the available style schemes will be reloaded next time the @manager is accessed.
  */
 void
