@@ -70,6 +70,7 @@ void     _gtk_source_view_assistants_size_allocate (GtkSourceViewAssistants *ass
 gboolean _gtk_source_view_assistants_handle_key    (GtkSourceViewAssistants *assistant,
                                                     guint                    keyval,
                                                     GdkModifierType          state);
+void     _gtk_source_view_assistants_update_all    (GtkSourceViewAssistants *assistants);
 
 void     _gtk_source_view_snippets_init        (GtkSourceViewSnippets *snippets,
                                                 GtkSourceView         *view);
@@ -86,6 +87,7 @@ gboolean _gtk_source_view_snippets_key_pressed (GtkSourceViewSnippets *snippets,
                                                 guint                  keycode,
                                                 GdkModifierType        state);
 
+void     _gtk_source_view_hide_completion                    (GtkSourceView *view);
 gboolean _gtk_source_view_get_current_line_background        (GtkSourceView *view,
                                                               GdkRGBA       *rgba);
 gboolean _gtk_source_view_get_current_line_number_background (GtkSourceView *view,
