@@ -23,6 +23,7 @@
 
 #include <glib/gi18n.h>
 
+#include "gtksourcecompletion-private.h"
 #include "gtksourceindenter.h"
 #include "gtksourceview.h"
 
@@ -134,6 +135,25 @@ gtk_source_vim_insert_handle_keypress (GtkSourceVimState *state,
 				g_signal_emit_by_name (view, "paste-clipboard");
 				return TRUE;
 
+			case GDK_KEY_n:
+			case GDK_KEY_p:
+			{
+				GtkSourceCompletion *completion = gtk_source_view_get_completion (view);
+
+				if (_gtk_source_completion_get_visible (completion))
+				{
+					_gtk_source_completion_move_cursor (completion,
+									    GTK_MOVEMENT_DISPLAY_LINES,
+									    keyval == GDK_KEY_n ? 1 : -1);
+				}
+				else
+				{
+					gtk_source_completion_show (completion);
+				}
+
+				return TRUE;
+			}
+
 			default:
 				break;
 		}
@@ -171,25 +191,38 @@ gtk_source_vim_insert_handle_event (GtkSourceVimState *state,
 	g_assert (GTK_SOURCE_IS_VIM_INSERT (self));
 	g_assert (event != NULL);
 
-	if (!(view = gtk_source_vim_state_get_view (state)))
-		return FALSE;
+	view = gtk_source_vim_state_get_view (state);
 
+	/* We only handle keypress, otherwise defer to the normal event processing
+	 * flow and/or input methods.
+	 */
+	if (view == NULL || gdk_event_get_event_type (event) != GDK_KEY_PRESS)
+	{
+		return FALSE;
+	}
+
+	/* gtk_text_view_im_context_filter_keypress() will always filter input that
+	 * can be converted into an GtkIMContext::commit emission so we must check
+	 * to see if any of our handlers will check first.
+	 *
+	 * This has a sort of annoying impact with the underlying input method that
+	 * we could collide, but there doesn't seem to be much we can do about that.
+	 *
+	 * https://gitlab.gnome.org/GNOME/gtk/-/issues/5349
+	 */
 	keyval = gdk_key_event_get_keyval (event);
 	keycode = gdk_key_event_get_keycode (event);
 	mods = gdk_event_get_modifier_state (event)
 	     & gtk_accelerator_get_default_mod_mask ();
 
-	/* Allow input methods to complete */
-	if (gtk_text_view_im_context_filter_keypress (GTK_TEXT_VIEW (view), event))
-		return TRUE;
-
-	/* Only deal with presses after this */
-	if (gdk_event_get_event_type (event) != GDK_KEY_PRESS)
-		return TRUE;
-
 	gtk_source_vim_state_keyval_to_string (keyval, mods, string);
 
-	return GTK_SOURCE_VIM_STATE_GET_CLASS (self)->handle_keypress (state, keyval, keycode, mods, string);
+	if (GTK_SOURCE_VIM_STATE_GET_CLASS (self)->handle_keypress (state, keyval, keycode, mods, string))
+	{
+		return TRUE;
+	}
+
+	return FALSE;
 }
 
 static void
@@ -470,6 +503,7 @@ gtk_source_vim_insert_dispose (GObject *object)
 	G_OBJECT_CLASS (gtk_source_vim_insert_parent_class)->dispose (object);
 
 	g_clear_pointer (&self->prefix, g_free);
+	g_clear_pointer (&self->suffix, g_free);
 	gtk_source_vim_state_release (&self->history);
 	gtk_source_vim_state_release (&self->motion);
 	gtk_source_vim_state_release (&self->selection_motion);
@@ -532,12 +566,8 @@ gtk_source_vim_insert_set_prefix (GtkSourceVimInsert *self,
 {
 	g_return_if_fail (GTK_SOURCE_IS_VIM_INSERT (self));
 
-	if (g_strcmp0 (self->prefix, prefix) != 0)
-	{
-		g_free (self->prefix);
-		self->prefix = g_strdup (prefix);
+	if (g_set_str (&self->prefix, prefix))
 		g_object_notify_by_pspec (G_OBJECT (self), properties [PROP_PREFIX]);
-	}
 }
 
 void
@@ -546,12 +576,8 @@ gtk_source_vim_insert_set_suffix (GtkSourceVimInsert *self,
 {
 	g_return_if_fail (GTK_SOURCE_IS_VIM_INSERT (self));
 
-	if (g_strcmp0 (self->suffix, suffix) != 0)
-	{
-		g_free (self->suffix);
-		self->suffix = g_strdup (suffix);
+	if (g_set_str (&self->suffix, suffix))
 		g_object_notify_by_pspec (G_OBJECT (self), properties [PROP_SUFFIX]);
-	}
 }
 
 void
