@@ -10,7 +10,9 @@ xxx${var\
 
 xxx$0000 # One digit parameter
 xxx$-xxx xxx$$xxx xxx$@xxx # Special parameters
+xxx$_param-xxx # Starting with '_'
 xxx${!array[@]}xxx${#array[-1]}xxx${array[0x1+var/2*$(cmd)]/a/b}xxx # Arrays
+xxx${array['key']}xxx${array["key"]} # Associative arrays
 
 xxx${parameter:-word}xxx${parameter-word}xxx # Use Default Values
 xxx${parameter:=word}xxx${parameter=word}xxx # Assign Default Values
@@ -87,6 +89,13 @@ EOF
 var1=$((var2+2#101+$(cmd)+($var3+ \
   "$var4")/0x1f))
 let var1='1'+010-23+`cmd`+var2 # Comment
+((${#arr[@]} == size))
+((a + b['key'] + c[0] == d))
+
+# Nested subshells (should not match arithmetic evaluation)
+(([ -f f ] && [ -f g ]) || ([ -f h ] && [ -f i ]))
+((command1 && command2) \
+ || (command3 && command4))
 
 # History expansion
 "txt!cmd:^-$:r:gs/xxx/yyy/:ptxt"
