@@ -22,11 +22,13 @@
 #include <string.h>
 #include <gtksourceview/gtksource.h>
 
+#define TEST_TYPE_ANNOTATION_PROVIDER (test_annotation_provider_get_type())
 #define TEST_TYPE_WIDGET (test_widget_get_type())
 #define TEST_TYPE_HOVER_PROVIDER (test_hover_provider_get_type())
 
 G_DECLARE_FINAL_TYPE (TestWidget, test_widget, TEST, WIDGET, GtkGrid)
 G_DECLARE_FINAL_TYPE (TestHoverProvider, test_hover_provider, TEST, HOVER_PROVIDER, GObject)
+G_DECLARE_FINAL_TYPE (TestAnnotationProvider, test_annotation_provider, TEST, ANNOTATION_PROVIDER, GtkSourceAnnotationProvider)
 
 struct _TestWidget
 {
@@ -46,11 +48,13 @@ struct _TestWidget
 	GtkLabel *cursor_position_info;
 	GtkSourceStyleSchemeChooserButton *chooser_button;
 	GtkIMContext *vim_im_context;
-	GtkComboBoxText *background_pattern;
+	GtkDropDown *background_pattern;
 	GtkWidget *top;
 	GtkScrolledWindow *scrolledwindow1;
 	GtkEventController *vim_controller;
 	GtkLabel *command_bar;
+	GtkSourceAnnotationProvider *annotation_provider;
+	GtkSourceAnnotationProvider *error_provider;
 };
 
 struct _TestHoverProvider
@@ -58,17 +62,22 @@ struct _TestHoverProvider
 	GObject parent_instance;
 };
 
+struct _TestAnnotationProvider
+{
+	GObject parent_instance;
+};
+
 static void hover_provider_iface_init (GtkSourceHoverProviderInterface *iface);
 
 G_DEFINE_TYPE (TestWidget, test_widget, GTK_TYPE_GRID)
 G_DEFINE_TYPE_WITH_CODE (TestHoverProvider, test_hover_provider, G_TYPE_OBJECT,
 			 G_IMPLEMENT_INTERFACE (GTK_SOURCE_TYPE_HOVER_PROVIDER, hover_provider_iface_init))
+G_DEFINE_TYPE (TestAnnotationProvider, test_annotation_provider, GTK_SOURCE_TYPE_ANNOTATION_PROVIDER)
 
 #define MARK_TYPE_1      "one"
 #define MARK_TYPE_2      "two"
 
 static GMainLoop *main_loop;
-static gchar *last_dir;
 static const char *cmd_filename;
 
 static void
@@ -315,6 +324,15 @@ right_margin_position_value_changed_cb (TestWidget    *self,
 	gtk_widget_queue_resize (GTK_WIDGET (self->map));
 }
 
+static void
+bottom_margin_value_changed_cb (TestWidget    *self,
+			        GtkSpinButton *button)
+{
+	gint margin = gtk_spin_button_get_value_as_int (button);
+
+	gtk_text_view_set_bottom_margin (GTK_TEXT_VIEW (self->view), margin);
+}
+
 static void
 highlight_syntax_toggled_cb (TestWidget     *self,
 			     GtkCheckButton *button)
@@ -386,13 +404,14 @@ update_indent_width (TestWidget *self)
 }
 
 static void
-smart_home_end_changed_cb (TestWidget  *self,
-			   GtkComboBox *combo)
+smart_home_end_selected_notify_cb (TestWidget  *self,
+				   GParamSpec  *pspec,
+				   GtkDropDown *drop_down)
 {
 	GtkSourceSmartHomeEndType type;
-	gint active = gtk_combo_box_get_active (combo);
+	guint selected = gtk_drop_down_get_selected (drop_down);
 
-	switch (active)
+	switch (selected)
 	{
 		case 0:
 			type = GTK_SOURCE_SMART_HOME_END_DISABLED;
@@ -465,73 +484,70 @@ forward_string_clicked_cb (TestWidget *self)
 }
 
 static void
-on_chooser_response_cb (TestWidget           *self,
-                        gint                  response,
-                        GtkFileChooserDialog *chooser)
+on_file_dialog_response_cb (GObject      *source,
+                            GAsyncResult *result,
+                            gpointer      user_data)
 {
+	TestWidget *self = user_data;
+	GtkFileDialog *dialog = GTK_FILE_DIALOG (source);
+	GError *error = NULL;
+	GFile *file;
+
 	g_assert (TEST_IS_WIDGET (self));
-	g_assert (GTK_IS_FILE_CHOOSER_DIALOG (chooser));
+	g_assert (GTK_IS_FILE_DIALOG (dialog));
 
-	if (response == GTK_RESPONSE_OK)
+	file = gtk_file_dialog_open_finish (dialog, result, &error);
+
+	if (file != NULL)
 	{
-		GFile *folder;
-		GFile *file;
-
-		file = gtk_file_chooser_get_file (GTK_FILE_CHOOSER (chooser));
-		folder = gtk_file_chooser_get_current_folder (GTK_FILE_CHOOSER (chooser));
-
-		if (file != NULL && folder != NULL)
-		{
-			g_free (last_dir);
-			last_dir = g_file_get_path (folder);
-			open_file (self, file);
-		}
-
-		g_clear_object (&folder);
-		g_clear_object (&file);
+		open_file (self, file);
+		g_object_unref (file);
 	}
-
-	gtk_window_destroy (GTK_WINDOW (chooser));
 }
 
 static void
 open_button_clicked_cb (TestWidget *self)
 {
 	GtkWidget *main_window;
-	GtkWidget *chooser;
+	GtkFileDialog *dialog;
 
 	main_window = GTK_WIDGET (gtk_widget_get_root (GTK_WIDGET (self->view)));
 
-	chooser = gtk_file_chooser_dialog_new ("Open file...",
-					       GTK_WINDOW (main_window),
-					       GTK_FILE_CHOOSER_ACTION_OPEN,
-					       "Cancel", GTK_RESPONSE_CANCEL,
-					       "Open", GTK_RESPONSE_OK,
-					       NULL);
+	dialog = gtk_file_dialog_new ();
 
-	if (last_dir == NULL)
-	{
-		last_dir = g_strdup (TOP_SRCDIR "/gtksourceview");
-	}
+	gtk_file_dialog_set_title (dialog, "Open file...");
+	gtk_file_dialog_set_accept_label (dialog, "Open");
 
-	if (last_dir != NULL && g_path_is_absolute (last_dir))
-	{
-		GFile *folder;
+	gtk_file_dialog_open (dialog,
+	                      GTK_WINDOW (main_window),
+	                      NULL,
+	                      on_file_dialog_response_cb,
+	                      self);
 
-		folder = g_file_new_for_path (last_dir);
-		gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (chooser), folder, NULL);
-		g_object_unref (folder);
-	}
-
-	g_signal_connect_object (chooser,
-				 "response",
-				 G_CALLBACK (on_chooser_response_cb),
-				 self,
-				 G_CONNECT_SWAPPED);
-
-	gtk_window_present (GTK_WINDOW (chooser));
+	g_object_unref (dialog);
 }
 
+static void
+markup_button_clicked_cb (TestWidget *self)
+{
+	GdkClipboard *clipboard;
+	GtkTextIter start, end;
+	char *markup;
+
+	if (!gtk_text_buffer_get_selection_bounds (GTK_TEXT_BUFFER (self->buffer), &start, &end))
+	{
+		gtk_text_buffer_get_bounds (GTK_TEXT_BUFFER (self->buffer), &start, &end);
+	}
+
+	markup = gtk_source_buffer_get_markup (self->buffer, &start, &end);
+
+	clipboard = gdk_display_get_clipboard (gtk_widget_get_display(GTK_WIDGET(self)));
+	gdk_clipboard_set_text (clipboard, markup);
+
+	g_free (markup);
+}
+
+
 #define NON_BLOCKING_PAGINATION
 
 #ifndef NON_BLOCKING_PAGINATION
@@ -936,14 +952,20 @@ add_source_mark_attributes (GtkSourceView *view)
 }
 
 static void
-on_background_pattern_changed (GtkComboBox *combobox,
-                               TestWidget  *self)
+background_pattern_selected_item_notify_cb (GtkDropDown *drop_down,
+					    GParamSpec  *pspec,
+					    TestWidget  *self)
 {
-	gchar *text;
+	GObject *selected_item;
+	const char *selected_item_str = NULL;
 
-	text = gtk_combo_box_text_get_active_text (GTK_COMBO_BOX_TEXT (combobox));
+	selected_item = gtk_drop_down_get_selected_item (drop_down);
+	if (selected_item != NULL)
+	{
+		selected_item_str = gtk_string_object_get_string (GTK_STRING_OBJECT (selected_item));
+	}
 
-	if (g_strcmp0 (text, "Grid") == 0)
+	if (g_strcmp0 (selected_item_str, "Grid") == 0)
 	{
 		gtk_source_view_set_background_pattern (self->view,
 		                                        GTK_SOURCE_BACKGROUND_PATTERN_TYPE_GRID);
@@ -953,8 +975,6 @@ on_background_pattern_changed (GtkComboBox *combobox,
 		gtk_source_view_set_background_pattern (self->view,
 		                                        GTK_SOURCE_BACKGROUND_PATTERN_TYPE_NONE);
 	}
-
-	g_free (text);
 }
 
 static void
@@ -990,6 +1010,97 @@ enable_hover_toggled_cb (TestWidget     *self,
 		gtk_source_hover_remove_provider (hover, test_hover_provider);
 }
 
+static void
+on_cursor_moved (TestWidget *self)
+{
+	GtkSourceAnnotation *annotation;
+	GtkTextBuffer *buffer;
+	GtkTextIter iter;
+	gint line;
+	gchar *line_text;
+
+	if (self->annotation_provider == NULL)
+	{
+		return;
+	}
+
+	buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (self->view));
+	gtk_text_buffer_get_iter_at_mark (buffer,
+	                                  &iter,
+	                                  gtk_text_buffer_get_insert (buffer));
+
+	line = gtk_text_iter_get_line (&iter);
+
+	line_text = g_strdup_printf ("Line %d annotation", line + 1);
+
+	annotation = gtk_source_annotation_new (line_text,
+	                                        g_icon_new_for_string ("dialog-information-symbolic", NULL),
+	                                        line,
+	                                        GTK_SOURCE_ANNOTATION_STYLE_NONE);
+
+	gtk_source_annotation_provider_remove_all (self->annotation_provider);
+
+	gtk_source_annotation_provider_add_annotation (self->annotation_provider, annotation);
+
+	g_object_unref (annotation);
+	g_free (line_text);
+}
+
+static void
+enable_annotations_toggled_cb (TestWidget     *self,
+                               GtkCheckButton *button)
+{
+	GtkSourceAnnotations *annotations = gtk_source_view_get_annotations (self->view);
+	gboolean enabled = gtk_check_button_get_active (button);
+
+	if (self->annotation_provider == NULL)
+	{
+		self->annotation_provider = GTK_SOURCE_ANNOTATION_PROVIDER (g_object_new (TEST_TYPE_ANNOTATION_PROVIDER, NULL));
+	}
+
+	if (self->error_provider == NULL)
+	{
+		GtkSourceAnnotation *annotation;
+
+		self->error_provider = GTK_SOURCE_ANNOTATION_PROVIDER (g_object_new (TEST_TYPE_ANNOTATION_PROVIDER, NULL));
+
+		annotation = gtk_source_annotation_new ("Error Style!",
+		                                        g_icon_new_for_string ("emblem-important-symbolic", NULL),
+		                                        23,
+		                                        GTK_SOURCE_ANNOTATION_STYLE_ERROR);
+
+		gtk_source_annotation_provider_add_annotation (self->error_provider, annotation);
+		g_object_unref (annotation);
+
+		annotation = gtk_source_annotation_new ("Warning Style!",
+		                                        g_icon_new_for_string ("dialog-warning-symbolic", NULL),
+		                                        25,
+		                                        GTK_SOURCE_ANNOTATION_STYLE_WARNING);
+
+		gtk_source_annotation_provider_add_annotation (self->error_provider, annotation);
+		g_object_unref (annotation);
+
+		annotation = gtk_source_annotation_new ("Accent Style without an icon",
+		                                        NULL,
+		                                        21,
+		                                        GTK_SOURCE_ANNOTATION_STYLE_ACCENT);
+
+		gtk_source_annotation_provider_add_annotation (self->error_provider, annotation);
+		g_object_unref (annotation);
+	}
+
+	if (enabled)
+	{
+		gtk_source_annotations_add_provider (annotations, self->annotation_provider);
+		gtk_source_annotations_add_provider (annotations, self->error_provider);
+	}
+	else
+	{
+		gtk_source_annotations_remove_provider (annotations, self->annotation_provider);
+		gtk_source_annotations_remove_provider (annotations, self->error_provider);
+	}
+}
+
 static void
 vim_checkbutton_toggled_cb (TestWidget     *self,
                             GtkCheckButton *button)
@@ -1050,12 +1161,14 @@ test_widget_class_init (TestWidgetClass *klass)
 
 	gtk_widget_class_bind_template_callback (widget_class, open_button_clicked_cb);
 	gtk_widget_class_bind_template_callback (widget_class, print_button_clicked_cb);
+	gtk_widget_class_bind_template_callback (widget_class, markup_button_clicked_cb);
 	gtk_widget_class_bind_template_callback (widget_class, highlight_syntax_toggled_cb);
 	gtk_widget_class_bind_template_callback (widget_class, highlight_matching_bracket_toggled_cb);
 	gtk_widget_class_bind_template_callback (widget_class, show_line_numbers_toggled_cb);
 	gtk_widget_class_bind_template_callback (widget_class, show_line_marks_toggled_cb);
 	gtk_widget_class_bind_template_callback (widget_class, show_right_margin_toggled_cb);
 	gtk_widget_class_bind_template_callback (widget_class, right_margin_position_value_changed_cb);
+	gtk_widget_class_bind_template_callback (widget_class, bottom_margin_value_changed_cb);
 	gtk_widget_class_bind_template_callback (widget_class, highlight_current_line_toggled_cb);
 	gtk_widget_class_bind_template_callback (widget_class, wrap_lines_toggled_cb);
 	gtk_widget_class_bind_template_callback (widget_class, auto_indent_toggled_cb);
@@ -1063,9 +1176,10 @@ test_widget_class_init (TestWidgetClass *klass)
 	gtk_widget_class_bind_template_callback (widget_class, tab_width_value_changed_cb);
 	gtk_widget_class_bind_template_callback (widget_class, backward_string_clicked_cb);
 	gtk_widget_class_bind_template_callback (widget_class, forward_string_clicked_cb);
-	gtk_widget_class_bind_template_callback (widget_class, smart_home_end_changed_cb);
+	gtk_widget_class_bind_template_callback (widget_class, smart_home_end_selected_notify_cb);
 	gtk_widget_class_bind_template_callback (widget_class, enable_snippets_toggled_cb);
 	gtk_widget_class_bind_template_callback (widget_class, enable_hover_toggled_cb);
+	gtk_widget_class_bind_template_callback (widget_class, enable_annotations_toggled_cb);
 	gtk_widget_class_bind_template_callback (widget_class, vim_checkbutton_toggled_cb);
 
 	gtk_widget_class_bind_template_child (widget_class, TestWidget, view);
@@ -1207,8 +1321,8 @@ test_widget_init (TestWidget *self)
 	                        G_BINDING_SYNC_CREATE);
 
 	g_signal_connect (self->background_pattern,
-	                  "changed",
-	                  G_CALLBACK (on_background_pattern_changed),
+	                  "notify::selected-item",
+	                  G_CALLBACK (background_pattern_selected_item_notify_cb),
 	                  self);
 
 	space_drawer = gtk_source_view_get_space_drawer (self->view);
@@ -1216,6 +1330,13 @@ test_widget_init (TestWidget *self)
 				space_drawer, "enable-matrix",
 				G_BINDING_BIDIRECTIONAL | G_BINDING_SYNC_CREATE);
 
+	g_signal_connect_swapped (self->buffer,
+				"notify::cursor-position",
+				G_CALLBACK (on_cursor_moved),
+				self);
+
+	g_print("connected!!\n");
+
 	if (cmd_filename)
 		file = g_file_new_for_commandline_arg (cmd_filename);
 	else
@@ -1266,6 +1387,69 @@ test_hover_provider_init (TestHoverProvider *self)
 {
 }
 
+static void
+test_annotation_provider_dispose (GObject *object)
+{
+	G_OBJECT_CLASS (test_annotation_provider_parent_class)->dispose (object);
+}
+
+static void
+test_annotation_provider_populate_hover_async (GtkSourceAnnotationProvider  *self,
+                                               GtkSourceAnnotation          *annotation,
+                                               GtkSourceHoverDisplay        *display,
+                                               GCancellable                 *cancellable,
+                                               GAsyncReadyCallback           callback,
+                                               gpointer                      user_data)
+{
+	GTask *task = NULL;
+
+	g_assert (GTK_SOURCE_IS_ANNOTATION_PROVIDER (self));
+	g_assert (GTK_SOURCE_IS_HOVER_DISPLAY (display));
+	g_assert (!cancellable || G_IS_CANCELLABLE (cancellable));
+
+	task = g_task_new (self, cancellable, callback, user_data);
+	g_task_set_source_tag (task, gtk_source_annotation_provider_populate_hover_async);
+
+	gtk_source_hover_display_append (display,
+	                                 g_object_new (GTK_TYPE_LABEL,
+	                                               "label", gtk_source_annotation_get_description (annotation),
+	                                               "margin-start", 12,
+	                                               "margin-end", 12,
+	                                               NULL));
+
+	g_task_return_boolean (task, TRUE);
+
+	g_object_unref (task);
+}
+
+static gboolean
+test_annotation_provider_populate_hover_finish (GtkSourceAnnotationProvider  *self,
+                                                GAsyncResult                 *result,
+                                                GError                      **error)
+{
+	g_return_val_if_fail (GTK_SOURCE_IS_ANNOTATION_PROVIDER (self), FALSE);
+	g_return_val_if_fail (G_IS_ASYNC_RESULT (result), FALSE);
+
+	return g_task_propagate_boolean (G_TASK (result), error);
+}
+
+static void
+test_annotation_provider_class_init (TestAnnotationProviderClass *klass)
+{
+	GObjectClass *object_class = G_OBJECT_CLASS (klass);
+	GtkSourceAnnotationProviderClass *provider_class = GTK_SOURCE_ANNOTATION_PROVIDER_CLASS (klass);
+
+	object_class->dispose = test_annotation_provider_dispose;
+
+	provider_class->populate_hover_async = test_annotation_provider_populate_hover_async;
+	provider_class->populate_hover_finish = test_annotation_provider_populate_hover_finish;
+}
+
+static void
+test_annotation_provider_init (TestAnnotationProvider *self)
+{
+}
+
 static void
 setup_search_paths (void)
 {
@@ -1314,7 +1498,7 @@ main (int argc, char *argv[])
 	test_widget = test_widget_new ();
 	gtk_window_set_child (GTK_WINDOW (window), GTK_WIDGET (test_widget));
 
-	gtk_widget_show (window);
+	gtk_window_present (GTK_WINDOW (window));
 
 	g_main_loop_run (main_loop);
 
